//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UtCompany.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

tCompany::~tCompany()
{
        for(int i = 0; i < numVuelos; i++)
        {
                        delete vuelos.at(i);
        }
}

bool tCompany::insertaVuelo(tipoVuelo tv, const string& o, const string& d, const tHora& hs, const tHora& dur, const string& co, int IdV, int pre)
{
        if(numVuelos < MAX_VUELOS)
        {
                tVuelo* vuel;
                if(tv == normal)
                {
                        vuel = new tVueloNormal(o, d, hs, dur, co, IdV, pre);
                }
                else
                {
                        vuel = new tVueloBajoCoste(o, d, hs, dur, co, IdV, pre);
                }
                if(vuelos.insert(vuel))
                {
                        numVuelos++;
                        return true;
                }
                else
                {
                        delete vuel;
                        return false;
                }
        }
        else
        {
                return false;
        }
}

bool tCompany::eliminaVuelo(int idVuelo)
{
        tSize pos;

        if(buscaVuelo(idVuelo, pos))
        {
                delete vuelos.at(pos);
                vuelos.erase(pos);
                numVuelos--;
                return true;
        }
        else
        {
                return false;
        }
}

bool tCompany::hazReserva(int idVuelo, tLocalizador& loc)
{
        tSize pos;

        if(buscaVuelo(idVuelo, pos))
        {
                return vuelos.at(pos)->hazReserva(loc);
        }
        else return false;
}

bool tCompany::eliminaReserva(const tLocalizador& loc)
{
        tSize pos;

        if(buscaVuelo(loc.dameIdVuelo() , pos))
        {
                return vuelos.at(pos)->eliminaReserva(loc);
        }
        else return false;
}

bool tCompany::salvar(const string& nomb) const
{
        ofstream sal;
        sal.open(nomb.data(), ios::binary);
        if(sal.is_open())
        {
                sal.write((char *) & numVuelos, sizeof(int));
                for(int i = 0; i < numVuelos; i++)
                {
                        vuelos.at(i)->salvar(sal);
                }
        }
        sal.close();
        return sal;
}

bool tCompany::cargar(const string& nomb)
{
        ifstream ent;
        tipoVuelo tv;
        ent.open(nomb.data(), ios::binary);
        if(ent.is_open())
        {
                for(int i = 0; i < numVuelos; i++)
                {
                        delete vuelos.at(i);
                }
                vuelos.clear();
                tVuelo* aux;
                ent.read((char *) & numVuelos, sizeof(int));
                for(int i = 0; i < numVuelos; i++)
                {
                        ent.read((char *) & tv, sizeof(tipoVuelo));
                        if (tv == normal)
                        {
                                aux = new tVueloNormal();
                        }
                        else
                        {
                                aux = new tVueloBajoCoste();
                        }
                        aux->cargar(ent);
                        vuelos.insert(aux);
                }
        }
        ent.close();
        return ent;
}

bool tCompany::buscaVuelo(int idVuelo, tSize& pos)
{
        //return buscaVuelo(idVuelo, pos, 0, numVuelos - 1);
        if(!vuelos.empty())
        {
                bool resultado;
                tVuelo* aux = new tVueloNormal
                        (
                        vuelos.at(0)->getOrigen(),
                        vuelos.at(0)->getDestino(),
                        vuelos.at(0)->getSalida(),
                        vuelos.at(0)->getDuracion(),
                        vuelos.at(0)->getCia(),
                        idVuelo,
                        vuelos.at(0)->getPrecio()
                        );
                resultado = vuelos.find(aux, pos);
                delete aux;
                return resultado;
        }
        else return false;
}

/*bool tCompany::buscaVuelo(int idVuelo, tSize& pos, tSize ini, tSize fin)
{
        tSize medio = (ini + fin) / 2;
        if(vuelos[medio]->getId() == idVuelo)
        {
                pos = medio;
                return true;
        }
        else
        {
                if(ini == fin)
                {
                        return false;
                }
                if(vuelos[medio]->getId() < idVuelo)
                {
                        return buscaVuelo(idVuelo, pos, medio + 1, fin);
                }
                else
                {
                        return buscaVuelo(idVuelo, pos, ini, medio - 1);
                }
        }
}
  */
bool tCompany::buscaReserva(tLocalizador& loc)
{
        tSize pos;
        if(buscaVuelo(loc.dameIdVuelo(), pos))
        {
                if(vuelos.at(pos)->buscaReserva(loc.dameIdLocalizador(), loc))
                {
                        return true;
                }
                else return false;
        }
        else return false;
}

bool tCompany::facturar(tLocalizador& loc, tSize col, tSize fil)
{
        tSize pos;
        if(buscaVuelo(loc.dameIdVuelo(), pos))
        {
                if(vuelos.at(pos)->facturar(loc, col, fil))
                {
                        if(vuelos.at(pos)->cerrado())
                        {
                                eliminaVuelo(loc.dameIdVuelo());
                        }
                        return true;
                }
                else return false;
        }
        else return false;
}









